// File: SortInterfaceDemo.java
// program to illustrate a use for an interface
// when implementing a sorting algorithm

import java.io.*;

interface AnyType
{
	public boolean isGreaterThan(AnyType datum);
}

class IntegerType implements AnyType
{
	// this class implements an integer type
	private int number;

	// constructors
	IntegerType(){number = 0;}
	IntegerType(int i){number = i;}

	// implementation of isGreaterThan for integer comparison
	public boolean isGreaterThan(AnyType datum)
	{
		return (this.number > ((IntegerType)datum).number);
	}

	public int toInteger()
	{
		return number;
	}
}

class StringType implements AnyType
{
	// this class implements a string type
	private String word;

	// constructors
	StringType(){word = "";}
	StringType(String s){word = s;}

	// implementation of isGreaterThan for string comparison
	public boolean isGreaterThan(AnyType datum)
	{
		return (this.word.compareTo(((StringType)datum).word) > 0);
	}

	public String toString()
	{
		return word;
	}
}



class Sort
{
	public static void bubbleSort(AnyType[] array)
	{
		AnyType temporary;
		int numberOfItems = array.length;

		for (int pass=1; pass != numberOfItems; pass++)
		{
			for (int index=0; index != numberOfItems-pass; index++)
			{
				if (array[index].isGreaterThan(array[index+1]))
				{
					temporary = array[index];
					array[index] = array[index+1];
					array[index+1] = temporary;
				}
			}
		}
	}
}

class SortInterfaceDemo
{
	static final int ARRAY_SIZE = 5;
	static PrintWriter screen = new PrintWriter(System.out,true);
	
	public static void main(String[] args)
	{
		// initialize an array of five integers
		IntegerType[] numbers = new IntegerType[ARRAY_SIZE];

		numbers[0] = new IntegerType(70);
		numbers[1] = new IntegerType(91);
		numbers[2] = new IntegerType(-56);
		numbers[3] = new IntegerType(2);
		numbers[4] = new IntegerType(29);

		screen.println("numbers before being sorted");

		for (int index=0; index < ARRAY_SIZE; index++)
		{
			screen.print(numbers[index].toInteger()+"\t");
		}

		screen.println("\nnumbers after being sorted");

		Sort.bubbleSort(numbers);

		for (int index=0; index < ARRAY_SIZE; index++)
		{
			screen.print(numbers[index].toInteger()+"\t");
		}		
		
		// initialize an array of five strings
		StringType[] words = new StringType[5];
		
		words[0] = new StringType("polymorphism");
		words[1] = new StringType("superclass");
		words[2] = new StringType("subclass");
		words[3] = new StringType("inheritance");
		words[4] = new StringType("abstraction");

		screen.println("\n\nwords before being sorted");

		for (int index=0; index < ARRAY_SIZE; index++)
		{
			screen.print(words[index].toString()+"\t");
		}

		screen.println("\nwords after being sorted");

		Sort.bubbleSort(words);

		for (int index=0; index < ARRAY_SIZE; index++)
		{
			screen.print(words[index].toString()+"\t");
		}
		screen.println();
	}
}
